#ifndef EVENEMENT_H
#define EVENEMENT_H

struct IEvenement
{
   class ReinitialisationAutomatique {};
   class ReinitialisationManuelle  {};
   virtual bool attendre() const volatile = 0;
   virtual bool attendre(const int ms) const volatile = 0;
   virtual void provoquer() const volatile = 0;
   virtual void reinitialiser() const volatile = 0;
protected:
   friend class Evenement;
   virtual ~IEvenement() = 0;
};

class Evenement
{
public:
   typedef IEvenement::ReinitialisationAutomatique ReinitialisationAutomatique;
   typedef IEvenement::ReinitialisationManuelle ReinitialisationManuelle;
private:
   const volatile IEvenement *evenement_;
   static const volatile IEvenement *creer(ReinitialisationAutomatique);
   static const volatile IEvenement *creer(ReinitialisationManuelle);
   static void liberer(const volatile IEvenement *) throw();
public:
   template <class Categorie>
      Evenement(Categorie categ)
          : evenement_(creer(categ))
      {
      }
   ~Evenement() throw()
      { liberer(evenement_); }
   bool attendre() const volatile
      { return evenement_->attendre(); }
   bool attendre(const int ms) const volatile
      { return evenement_->attendre(ms); }
   void provoquer() const volatile
      { return evenement_->provoquer(); }
   void reinitialiser() const volatile
      { return evenement_->reinitialiser(); }
};

#endif
